
#import "RootViewController.h"
#import "SliderViewController.h"
#import "ModalViewController.h"
#import "MapViewController.h"

/*
 The implementation of the RootViewController class. Unlike C++ or many other object
 oriented languages, the @implementation symbol denotes that all the following
 method implementations belong to the above named class. Therefore, there is no need
 to use a scope resolution operator when writing out the method implementations.
 */

@implementation RootViewController

#pragma mark -
#pragma mark View lifecycle

// OPTIONAL METHOD
// This is called when our view is loaded on the user's screen.
- (void)viewDidLoad {
    [super viewDidLoad];
    
    // Here we create an array of strings that will hold the text
    // for each of the cells (rows) in our tableview.
    //
    // Notice the "@" symbol. This denotes an objective-c style string.
    // The @"" string does in fact get instantiated into an object.
    mMenuOptions = [[NSArray alloc] initWithObjects:
                    @"Slider Example", 
                    @"Modal Example", 
                    @"Map Example", 
                    nil];
}


#pragma mark -
#pragma mark Table view data source

// Number of "sections" in the table.
// Since we are just displaying a simple list we will always return "1".
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}


// This method tells the TableViewController how many rows it should display to the screen...
// In our example, it is simply the number of strings contained in the MenuOptions Array.
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return [mMenuOptions count];
}


// This method handles the creation of the cells at each row.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    // Here we create a special string that helps us identify TableViewCells that should be reused.
    // In our case it really doesn't matter much since our list is so short and the cells are so simple, but
    // in other cases where we have cells that contain complex elements such as buttons and images it is
    // much more efficient to "reuse" a cell rather than destroy and create them as they come on/off the screen.
    static NSString *CellIdentifier = @"Cell";
    
    // Get a reusable cell (a cell that has the "reusable" identifier set and has gone off-screen).
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    
    // If we didn't get one, create a new cell...
    if (cell == nil) 
    {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
    }
    
	// Configure the cell. Use the array of strings to populate the text label.
    NSString* cellText = [mMenuOptions objectAtIndex:indexPath.row];
    [cell.textLabel setText:cellText];
    
    return cell;
}

// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the specified item to be editable.
    return NO;
}


#pragma mark -
#pragma mark Table view delegate

// If a row of the table is selected, this method is called. We override the method here to
// let our TableViewController know how to handle the selections.
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
    // Using our enumeration defined in the interface file, we check to see which row was
    // selected...
	if(indexPath.row == SLIDER_EXAMPLE)
    {
        // In order to progess through the screens using a navigation controller we need to "push"
        // new screens on the stack. Since each screen (more technically: UIView) is controlled by
        // a view controller we need to push a view controller onto the stack.
        
        // First create the new view controller...
        SliderViewController *detailViewController = [[SliderViewController alloc] initWithNibName:@"SliderViewController" bundle:nil];
        
        // Push it onto the stack...
        [self.navigationController pushViewController:detailViewController animated:YES];
        
        // Release the memory. We gave ownership of this view controller to the "pushViewController" method so
        // we can safely relinquish our ownership here.
        [detailViewController release];
    }
    else if(indexPath.row == MODAL_EXAMPLE)
    {
        ModalViewController *detailViewController = [[ModalViewController alloc] initWithNibName:@"ModalViewController" bundle:nil];
        [self.navigationController pushViewController:detailViewController animated:YES];
        [detailViewController release];
    }
    else if(indexPath.row == MAP_EXAMPLE)
    {
        MapViewController *detailViewController = [[MapViewController alloc] initWithNibName:@"MapViewController" bundle:nil];
        [self.navigationController pushViewController:detailViewController animated:YES];
        [detailViewController release];
    }
    
}


#pragma mark -
#pragma mark Memory management

// Occasionally, devices run short and memory and need to close inactive applications or non-critical things.
// This method is called by the OS when memory is low and is basically a nice way of saying "release some 
// memory or I'm gonna destroy you."
- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Relinquish ownership any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
    // Relinquish ownership of anything that can be recreated in viewDidLoad or on demand.
    // For example: self.myOutlet = nil;
}


- (void)dealloc {
    [super dealloc];
    [mMenuOptions release];
}


@end

